local addon, GW = ...

--hold the user interface
GW.UI = {}
GW.UI.OPT = {}
GW.UI.SET = {}

--hold the buttons and the settings of the current Role
GW.buttons = {}
GW.settings = {}

--hold the update information from previus frames
GW.tryingToLoad = false
GW.enableUpdate = false
GW.countFixData = 0
GW.lastUpdate = 0

--hold the current tooltip
GW.tooltipID = nil

--hold the counter for running updates not every frame
GW.updateCounter = 0
GW.framesPerUpdate = 2

--hold maximum timers and buttons
GW.maxTimers = 5
GW.maxButtons = 12

--hold the user's current role ID
GW.currentRole = 0

--default settings
local default_settings = {
	width = 100, health = 15, mana = 5, target = 12, button = 24, timer = 12,
	buttons = 1, timers = 3, maxrange = 30,
	locked=false, visible=true, handleX = 50, handleY = 100, padding = 2,
	layout = 5, growthDir = 1, growthX = 1, growthY = 1, showPlayer=false, showPet=false,
	petX = 50, petY = 0, petVisible=false
}

GW.backupSettings = default_settings

--creates the user interface
function GW.buildUI()
	--contexts
	GW.context = UI.CreateContext(addon.identifier)
	GW.context:SetSecureMode("restricted")
	GW.secureContext = UI.CreateFrame("Texture", "GW.secureContext", GW.context)
	GW.secureContext:SetSecureMode("restricted")
	GW.secureContext:SetLayer(0)
	GW.frameContext = UI.CreateFrame("Texture", "GW.frameContext", GW.context)
	GW.frameContext:SetSecureMode("normal")
	GW.frameContext:SetLayer(20)
	
	--handle frame. For moving the frameset
	GW.UI.handle = UI.CreateFrame("Texture", "GW.UI.handle", GW.frameContext)
	GW.UI.handle:SetWidth(44)
	GW.UI.handle:SetHeight(44)
	GW.UI.handle:SetLayer(1)
	GW.UI.handle:SetPoint("TOPLEFT", UIParent, "TOPLEFT", GW.settings.handleX, GW.settings.handleY)
	GW.UI.handle:SetTexture(addon.identifier, "img/handle.png")
	--events for dragging
	GW.UI.handle:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if GW.settings.locked == false and not Inspect.System.Secure() then
			local mouseData = Inspect.Mouse()
			self.sx = mouseData.x - GW.UI.handle:GetLeft()
			self.sy = mouseData.y - GW.UI.handle:GetTop()
			self.MouseDown = true
		end
	end, "Event.UI.Input.Mouse.Left.Down")
	GW.UI.handle:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		if GW.settings.locked == false and not Inspect.System.Secure() then
			local mouseData = Inspect.Mouse()
			GW.settings.handleX = GW.UI.handle:GetLeft()
			GW.settings.handleY = GW.UI.handle:GetTop()
		end
		self.MouseDown = false
	end, "Event.UI.Input.Mouse.Left.Up")
	GW.UI.handle:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if GW.settings.locked == false and not Inspect.System.Secure() and self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			GW.UI.handle:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")
	GW.UI.handle:SetVisible(false)
	
	--pet handle frame. For moving the pet frame
	GW.UI.petHandle = UI.CreateFrame("Texture", "GW.UI.petHandle", GW.frameContext)
	GW.UI.petHandle:SetWidth(22)
	GW.UI.petHandle:SetHeight(22)
	GW.UI.petHandle:SetLayer(3)
	GW.UI.petHandle:SetPoint("TOPLEFT", UIParent, "TOPLEFT", GW.settings.petX, GW.settings.petY)
	GW.UI.petHandle:SetTexture(addon.identifier, "img/handle.png")
	--events for dragging
	GW.UI.petHandle:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if GW.settings.locked == false and not Inspect.System.Secure() then
			local mouseData = Inspect.Mouse()
			self.sx = mouseData.x - GW.UI.petHandle:GetLeft()
			self.sy = mouseData.y - GW.UI.petHandle:GetTop()
			self.MouseDown = true
		end
	end, "Event.UI.Input.Mouse.Left.Down")
	GW.UI.petHandle:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		if GW.settings.locked == false and not Inspect.System.Secure() then
			local mouseData = Inspect.Mouse()
			GW.settings.petX = GW.UI.petHandle:GetLeft()
			GW.settings.petY = GW.UI.petHandle:GetTop()
		end
		self.MouseDown = false
	end, "Event.UI.Input.Mouse.Left.Up")
	GW.UI.petHandle:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if GW.settings.locked == false and not Inspect.System.Secure() and self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			GW.UI.petHandle:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")
	GW.UI.petHandle:SetVisible(false)
	
	GW.UI.group = {} --the group frames
	GW.UI.width = GW.settings.width --width of one frame
	GW.UI.height = GW.makeHeight() --height of one frame
	
	--create 20 frames
	for i=1,20 do
		--create the frame
		local unit = "group"..i
		if i < 10 then unit = "group0"..i end
		local newFrame = GW.createFrame(unit)
		
		--custom function to set whether this frame is enabled or not.
		--if we don't move it out of the way, then visible or not, it will consume macro clicks.
		function newFrame:SetEnabled(enabled)
			if Inspect.System.Secure() then return end
			local different = (not (enabled == self.isEnabled))
			self.isEnabled = enabled
			if different then
				GW.updateFramePosition(i)
			end
		end
		function newFrame:GetEnabled()
			return self.isEnabled
		end
		
		--save the frame
		GW.UI.group[i] = newFrame
	end
	
	--player frame for when not in group
	GW.UI.player = GW.createFrame("player")
	GW.UI.group[21] = GW.UI.player --for easier looping
	--custom function to set whether this frame is enabled or not.
	--if we don't move it out of the way, then visible or not, it will consume macro clicks.
	function GW.UI.player:SetEnabled(enabled)
		if Inspect.System.Secure() then return end
		self.isEnabled = enabled
		GW.UI.player.frame:ClearPoint("TOPLEFT")
		GW.UI.player.frame:ClearPoint("TOPRIGHT")
		GW.UI.player.frame:ClearPoint("BOTTOMLEFT")
		GW.UI.player.frame:ClearPoint("BOTTOMRIGHT")
		if enabled then
			GW.makeAnchor()
			GW.UI.player.frame:SetPoint(GW.anchor, GW.UI.handle, "CENTER", 0, 0)
		else
			GW.UI.player.frame:SetPoint("TOPLEFT", GW.UI.handle, "CENTER", 999999, 999999)
		end
	end
	function GW.UI.player:GetEnabled()
		return self.isEnabled
	end
	--pet frame for when not in group
	
	GW.UI.pet = GW.createFrame("player.pet")
	GW.UI.group[22] = GW.UI.pet --for easier looping
	--custom function to set whether this frame is enabled or not.
	--if we don't move it out of the way, then visible or not, it will consume macro clicks.
	function GW.UI.pet:SetEnabled(enabled)
		if Inspect.System.Secure() then return end
		self.isEnabled = enabled
		GW.UI.pet.frame:ClearPoint("TOPLEFT")
		GW.UI.pet.frame:ClearPoint("TOPRIGHT")
		GW.UI.pet.frame:ClearPoint("BOTTOMLEFT")
		GW.UI.pet.frame:ClearPoint("BOTTOMRIGHT")
		if enabled then
			GW.makeAnchor()
			GW.UI.pet.frame:SetPoint(GW.anchor, GW.UI.petHandle, "CENTER", 0, 0)
		else
			GW.UI.pet.frame:SetPoint("TOPLEFT", GW.UI.petHandle, "CENTER", 999999, 999999)
		end
	end
	function GW.UI.pet:GetEnabled()
		return self.isEnabled
	end
end

--creates a frame for a single party member
--i: slot of this party member 1-20 (ex: 1 for group01)
function GW.createFrame(unit)
	local newFrame = {} -- a new frame to return
	--------------------------------------------------------------------------------------------
	-- Main frame. Represents a party member. All following frames are attached to this. -------
	--------------------------------------------------------------------------------------------
	newFrame.frame = UI.CreateFrame("Frame", "GW.UI.frame."..unit, GW.frameContext)
	newFrame.frame:SetWidth(GW.UI.width)
	newFrame.frame:SetHeight(GW.UI.height)
	newFrame.frame:SetLayer(2)
	if unit == "player.pet" then
		newFrame.frame:SetLayer(4)
	end
	
	--frame background texture.
	newFrame.frameBG = UI.CreateFrame("Texture", "GW.UI.frameBG."..unit, newFrame.frame)
	newFrame.frameBG:SetLayer(1)
	newFrame.frameBG:SetTexture(addon.identifier, "img/frame.png")
	newFrame.frameBG:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", 0, 0)
	newFrame.frameBG:SetPoint("BOTTOMRIGHT", newFrame.frame, "BOTTOMRIGHT", 0, 0)
	
	-- Name of the party member.
	newFrame.name = UI.CreateFrame("Text", "GW.UI.name."..unit, newFrame.frame)
	newFrame.name:SetWidth((GW.settings.width-(2*GW.settings.padding)))
	newFrame.name:SetHeight(GW.settings.health+GW.settings.mana)
	newFrame.name:SetLayer(50)
	newFrame.name:SetFontSize(((GW.settings.health+GW.settings.mana)/2)+2)
	newFrame.name:SetFontColor(1,1,1)
	newFrame.name:SetEffectGlow({strength=10, blurX=1, blurY=1, offsetX=1, offsetY=1})
	newFrame.name:SetText("")
	newFrame.name:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", GW.settings.padding, GW.settings.padding)
	
	--role icon texture
	newFrame.role = UI.CreateFrame("Texture", "GW.UI.role."..unit, newFrame.frame)
	newFrame.role:SetLayer(60)
	newFrame.role:SetTexture("Rift", GW.icons["dps"])
	newFrame.role:SetPoint("TOPRIGHT", newFrame.name, "TOPRIGHT", 0, 0)
	newFrame.role:SetPoint("BOTTOMRIGHT", newFrame.name, "BOTTOMRIGHT", 0, 0)
	newFrame.role:SetWidth(newFrame.role:GetHeight())
	
	-- health frame. RESTRICTED. Handles macros for targeting this party member.
	newFrame.health = UI.CreateFrame("Texture", "GW.UI.health."..unit, GW.secureContext)
	newFrame.health:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.health:SetHeight(GW.settings.health)
	newFrame.health:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", GW.settings.padding, GW.settings.padding)
	newFrame.health:SetSecureMode("restricted")
	newFrame.health:SetMouseoverUnit(unit)
	newFrame.health:EventMacroSet(Event.UI.Input.Mouse.Left.Click, GW.fixMacro("target @unit",unit))
	
	-- healthBG frame. A background texture to make the health look nice when depleted.
	-- this frame has the added benefit of always being the same size, and not restricted.
	newFrame.healthBG = UI.CreateFrame("Texture", "GW.UI.healthBG."..unit, newFrame.frame)
	newFrame.healthBG:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.healthBG:SetHeight(GW.settings.health)
	newFrame.healthBG:SetLayer(10)
	newFrame.healthBG:SetTexture(addon.identifier, "img/barbg.png")
	newFrame.healthBG:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", GW.settings.padding, GW.settings.padding)
	
	-- healthbar frame. The width of this frame will change depending on the party member's health.
	newFrame.healthbar = UI.CreateFrame("Texture", "GW.UI.healthbar."..unit, newFrame.frame)
	newFrame.healthbar:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.healthbar:SetHeight(GW.settings.health)
	newFrame.healthbar:SetLayer(20)
	newFrame.healthbar:SetTexture(addon.identifier, "img/bar.png")
	newFrame.healthbar:SetBackgroundColor(0, 1, 0)
	newFrame.healthbar:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", GW.settings.padding, GW.settings.padding)
	
	-- absorbbar frame. The width of this frame will change depending on the party member's damage absorption
	newFrame.absorbbar = UI.CreateFrame("Texture", "GW.UI.absorbbar."..unit, newFrame.frame)
	newFrame.absorbbar:SetWidth(0)
	newFrame.absorbbar:SetHeight(GW.settings.health)
	newFrame.absorbbar:SetLayer(30)
	newFrame.absorbbar:SetTexture(addon.identifier, "img/barAbsorb.png")
	newFrame.absorbbar:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", GW.settings.padding, GW.settings.padding)
	
	-- mana frame. RESTRICTED. Handles macros for targeting this party member.
	newFrame.mana = UI.CreateFrame("Texture", "GW.UI.mana."..unit, GW.secureContext)
	newFrame.mana:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.mana:SetHeight(GW.settings.mana)
	newFrame.mana:SetPoint("TOPLEFT", newFrame.healthBG, "BOTTOMLEFT", 0, 0)
	newFrame.mana:SetSecureMode("restricted")
	newFrame.mana:SetMouseoverUnit(unit)
	newFrame.mana:EventMacroSet(Event.UI.Input.Mouse.Left.Click, GW.fixMacro("target @unit",unit))
	
	-- manaBG frame. A background texture to make the mana look nice when depleted.
	-- this frame has the added benefit of always being the same size, and not restricted.
	newFrame.manaBG = UI.CreateFrame("Texture", "GW.UI.manaBG."..unit, newFrame.frame)
	newFrame.manaBG:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.manaBG:SetHeight(GW.settings.mana)
	newFrame.manaBG:SetLayer(10)
	newFrame.manaBG:SetTexture(addon.identifier, "img/barbg.png")
	newFrame.manaBG:SetPoint("TOPLEFT", newFrame.healthBG, "BOTTOMLEFT", 0, 0)
	
	-- manabar frame. The width of this frame will change depending on the party member's mana.
	newFrame.manabar = UI.CreateFrame("Texture", "GW.UI.manabar."..unit, newFrame.frame)
	newFrame.manabar:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.manabar:SetHeight(GW.settings.mana)
	newFrame.manabar:SetLayer(20)
	newFrame.manabar:SetTexture(addon.identifier, "img/bar.png")
	newFrame.manabar:SetBackgroundColor(0, 0, 1)
	newFrame.manabar:SetPoint("TOPLEFT", newFrame.healthBG, "BOTTOMLEFT", 0, 0)
	
	--------------------------------------------------------------------------------------------
	-- Timerbars. These can be clicked to cast spells, and will show remaining time ------------
	-- of buffs caused by the left-click spell on the unit this frame belongs to.   ------------
	--------------------------------------------------------------------------------------------
	newFrame.timer = {}
	newFrame.timerBG = {}
	newFrame.timerHover = {}
	newFrame.timerbar = {}
	for n=1,GW.maxTimers do
		--------------------------------------------------------------------------------------------
		-- timer frame. RESTRICTED. invisible frame which handles macro clicks. --------------------
		--------------------------------------------------------------------------------------------
		newFrame.timer[n] = UI.CreateFrame("Texture", "GW.UI.timer."..unit.."."..n, GW.secureContext)
		newFrame.timer[n]:SetWidth(GW.settings.width-(2*GW.settings.padding))
		newFrame.timer[n]:SetHeight(GW.settings.timer)
		-- attach it to mana (first timer) or the previous timer
		if n == 1 then
			newFrame.timer[n]:SetPoint("TOPLEFT", newFrame.manaBG, "BOTTOMLEFT", 0, 0)
		else
			newFrame.timer[n]:SetPoint("TOPLEFT", newFrame.timerBG[n-1], "BOTTOMLEFT", 0, 0)
		end
		newFrame.timer[n]:SetSecureMode("restricted")
		newFrame.timer[n]:SetMouseMasking("limited")
		
		--------------------------------------------------------------------------------------------
		-- timerBG frame. Texture frame for each timerbar that also handles tooltips. --------------
		--------------------------------------------------------------------------------------------
		newFrame.timerBG[n] = UI.CreateFrame("Texture", "GW.UI.timerBG."..unit.."."..n, newFrame.frame)
		newFrame.timerBG[n]:SetWidth(GW.settings.width-(2*GW.settings.padding))
		newFrame.timerBG[n]:SetHeight(GW.settings.timer)
		newFrame.timerBG[n]:SetLayer(10)
		newFrame.timerBG[n]:SetMouseMasking("limited")
		newFrame.timerBG[n]:SetTexture(addon.identifier, "img/barbg.png")
		-- attach it to mana (first timer) or the previous timer
		if n == 1 then
			newFrame.timerBG[n]:SetPoint("TOPLEFT", newFrame.manaBG, "BOTTOMLEFT", 0, 0)
		else
			newFrame.timerBG[n]:SetPoint("TOPLEFT", newFrame.timerBG[n-1], "BOTTOMLEFT", 0, 0)
		end
		-- set the tooltip handlers
		newFrame.timerBG[n]:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self)
			if GW.buttons.timers[n] then
				local show = GW.getShow(GW.buttons.timers[n].left)
				if show ~= nil then
					GW.tooltipID = show
					Command.Tooltip(show)
				end
				newFrame.timerHover[n]:SetTexture(addon.identifier, "img/barhover.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.In")
		newFrame.timerBG[n]:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self)
			if GW.buttons.timers[n] then
				if GW.tooltipID == GW.getShow(GW.buttons.timers[n].left) then
					Command.Tooltip(nil)
				end
				newFrame.timerHover[n]:SetTexture(addon.identifier, "img/nothing.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.Out")
		
		--------------------------------------------------------------------------------------------
		-- timerHover frame. Texture frame which changes width based on time remaining. --------------
		--------------------------------------------------------------------------------------------
		newFrame.timerHover[n] = UI.CreateFrame("Texture", "GW.UI.timerHover."..unit.."."..n, newFrame.frame)
		newFrame.timerHover[n]:SetWidth(0)
		newFrame.timerHover[n]:SetHeight(GW.settings.timer)
		newFrame.timerHover[n]:SetLayer(25)
		newFrame.timerHover[n]:SetTexture(addon.identifier, "img/nothing.png")
		-- attach it to mana (first timer) or the previous timer
		if n == 1 then
			newFrame.timerHover[n]:SetPoint("TOPLEFT", newFrame.manaBG, "BOTTOMLEFT", 0, 0)
		else
			newFrame.timerHover[n]:SetPoint("TOPLEFT", newFrame.timerBG[n-1], "BOTTOMLEFT", 0, 0)
		end
		
		--------------------------------------------------------------------------------------------
		-- timerbar frame. Texture frame which changes width based on time remaining. --------------
		--------------------------------------------------------------------------------------------
		newFrame.timerbar[n] = UI.CreateFrame("Texture", "GW.UI.timerbar."..unit.."."..n, newFrame.frame)
		newFrame.timerbar[n]:SetWidth(0)
		newFrame.timerbar[n]:SetHeight(GW.settings.timer)
		newFrame.timerbar[n]:SetLayer(20)
		newFrame.timerbar[n]:SetTexture(addon.identifier, "img/bar.png")
		-- attach it to mana (first timer) or the previous timer
		if n == 1 then
			newFrame.timerbar[n]:SetPoint("TOPLEFT", newFrame.manaBG, "BOTTOMLEFT", 0, 0)
		else
			newFrame.timerbar[n]:SetPoint("TOPLEFT", newFrame.timerBG[n-1], "BOTTOMLEFT", 0, 0)
		end
	end
	
	--------------------------------------------------------------------------------------------
	-- Buttons. These can be clicked to cast spells, and will display the texture --------------
	-- of the spell bound to the left-click no-modifier spell.                    --------------
	--------------------------------------------------------------------------------------------
	newFrame.button = {}
	newFrame.buttonBG = {}
	newFrame.buttonBorder = {}
	newFrame.buttonHover = {}
	newFrame.buttonbar = {}
	for n=1,GW.maxButtons do
		--------------------------------------------------------------------------------------------
		-- button frame. RESTRICTED. invisible frame which handles macro clicks. -------------------
		--------------------------------------------------------------------------------------------
		newFrame.button[n] = UI.CreateFrame("Texture", "GW.UI.button."..unit.."."..n, GW.secureContext)
		newFrame.button[n]:SetWidth(GW.settings.button)
		newFrame.button[n]:SetHeight(GW.settings.button)
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.button[n]:SetPoint("TOPLEFT", newFrame.timerBG[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.button[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.button[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
		newFrame.button[n]:SetSecureMode("restricted")
		newFrame.button[n]:SetMouseMasking("limited")
		
		--------------------------------------------------------------------------------------------
		-- buttonBG frame. Texture frame for each button that also handles tooltips. ---------------
		--------------------------------------------------------------------------------------------
		newFrame.buttonBG[n] = UI.CreateFrame("Texture", "GW.UI.buttonBG."..unit.."."..n, newFrame.frame)
		newFrame.buttonBG[n]:SetWidth(GW.settings.button)
		newFrame.buttonBG[n]:SetHeight(GW.settings.button)
		newFrame.buttonBG[n]:SetLayer(10)
		newFrame.buttonBG[n]:SetMouseMasking("limited")
		newFrame.buttonBG[n]:SetTexture(addon.identifier, "img/buttonbg.png")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonBG[n]:SetPoint("TOPLEFT", newFrame.timerBG[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonBG[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonBG[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
		-- set the tooltip handlers
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self)
			if GW.buttons.buttons[n] then
				local show = GW.getShow(GW.buttons.buttons[n].left)
				if show ~= nil then
					GW.tooltipID = show
					Command.Tooltip(show)
				end
				newFrame.buttonHover[n]:SetTexture(addon.identifier, "img/buttonhover.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.In")
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self)
			if GW.buttons.buttons[n] then
				if GW.tooltipID == GW.getShow(GW.buttons.buttons[n].left) then
					Command.Tooltip(nil)
				end
				newFrame.buttonHover[n]:SetTexture(addon.identifier, "img/nothing.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.Out")
		
		--------------------------------------------------------------------------------------------
		-- buttonBorder frame. Texture frame with nice button border and no other purpose. ---------
		--------------------------------------------------------------------------------------------
		newFrame.buttonBorder[n] = UI.CreateFrame("Texture", "GW.UI.buttonBorder."..unit.."."..n, newFrame.frame)
		newFrame.buttonBorder[n]:SetWidth(GW.settings.button)
		newFrame.buttonBorder[n]:SetHeight(GW.settings.button)
		newFrame.buttonBorder[n]:SetLayer(20)
		newFrame.buttonBorder[n]:SetTexture(addon.identifier, "img/button.png")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonBorder[n]:SetPoint("TOPLEFT", newFrame.timerBG[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonBorder[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonBorder[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
		
		--------------------------------------------------------------------------------------------
		-- buttonHover frame. Texture frame for hovering highlight and no other purpose. -----------
		--------------------------------------------------------------------------------------------
		newFrame.buttonHover[n] = UI.CreateFrame("Texture", "GW.UI.buttonHover."..unit.."."..n, newFrame.frame)
		newFrame.buttonHover[n]:SetWidth(GW.settings.button)
		newFrame.buttonHover[n]:SetHeight(GW.settings.button)
		newFrame.buttonHover[n]:SetLayer(25)
		newFrame.buttonHover[n]:SetTexture(addon.identifier, "img/nothing.png")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonHover[n]:SetPoint("TOPLEFT", newFrame.timerBG[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonHover[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonHover[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
		
		
		--------------------------------------------------------------------------------------------
		-- buttonbar frame. Overlay frame which changes height based on cooldown. ------------------
		--------------------------------------------------------------------------------------------
		newFrame.buttonbar[n] = UI.CreateFrame("Texture", "GW.UI.buttonbar."..unit.."."..n, newFrame.frame)
		
		-- only set width if this button doesn't exceed frame
		newFrame.buttonbar[n]:SetWidth(GW.settings.button)
		newFrame.buttonbar[n]:SetHeight(0)
		newFrame.buttonbar[n]:SetLayer(30)
		newFrame.buttonbar[n]:SetAlpha(0.75)
		newFrame.buttonbar[n]:SetTexture(addon.identifier, "img/overlay.png")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonbar[n]:SetPoint("TOPLEFT", newFrame.timerBG[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonbar[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonbar[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
	end
	
	local n = (math.floor(GW.maxButtons/2)+1)
	
	-- target frame. RESTRICTED. Handles macros for targeting this party member's target.
	newFrame.target = UI.CreateFrame("Texture", "GW.UI.target."..unit, GW.secureContext)
	newFrame.target:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.target:SetHeight(GW.settings.target)
	newFrame.target:SetPoint("TOPLEFT", newFrame.buttonBG[n], "BOTTOMLEFT", 0, 0)
	newFrame.target:SetSecureMode("restricted")
	newFrame.target:SetMouseoverUnit(unit..".target")
	newFrame.target:EventMacroSet(Event.UI.Input.Mouse.Left.Click, GW.fixMacro("target @unitt",unit))
	
	-- targetBG frame. A background texture to make the target's health look nice when depleted.
	-- this frame has the added benefit of always being the same size, and not restricted.
	newFrame.targetBG = UI.CreateFrame("Texture", "GW.UI.targetBG."..unit, newFrame.frame)
	newFrame.targetBG:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.targetBG:SetHeight(GW.settings.target)
	newFrame.targetBG:SetLayer(10)
	newFrame.targetBG:SetTexture(addon.identifier, "img/barbg.png")
	newFrame.targetBG:SetPoint("TOPLEFT", newFrame.buttonBG[n], "BOTTOMLEFT", 0, 0)
	
	-- targetbar frame. The width of this frame will change depending on the party member's target's mana.
	newFrame.targetbar = UI.CreateFrame("Texture", "GW.UI.targetbar."..unit, newFrame.frame)
	newFrame.targetbar:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.targetbar:SetHeight(GW.settings.target)
	newFrame.targetbar:SetLayer(20)
	newFrame.targetbar:SetTexture(addon.identifier, "img/bar.png")
	newFrame.targetbar:SetBackgroundColor(1, 0, 0)
	newFrame.targetbar:SetPoint("TOPLEFT", newFrame.buttonBG[n], "BOTTOMLEFT", 0, 0)
	
	-- targetbar frame. The width of this frame will change depending on the party member's target's mana.
	newFrame.tabsorbbar = UI.CreateFrame("Texture", "GW.UI.tabsorbbar."..unit, newFrame.frame)
	newFrame.tabsorbbar:SetWidth(0)
	newFrame.tabsorbbar:SetHeight(GW.settings.target)
	newFrame.tabsorbbar:SetLayer(30)
	newFrame.tabsorbbar:SetTexture(addon.identifier, "img/barAbsorb.png")
	newFrame.tabsorbbar:SetPoint("TOPLEFT", newFrame.buttonBG[n], "BOTTOMLEFT", 0, 0)
	
	-- Name of the target.
	newFrame.targetname = UI.CreateFrame("Text", "GW.UI.targetname."..unit, newFrame.frame)
	newFrame.targetname:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.targetname:SetHeight(GW.settings.target)
	newFrame.targetname:SetLayer(50)
	newFrame.targetname:SetFontSize((GW.settings.target/2)+2)
	newFrame.targetname:SetFontColor(1,1,1)
	newFrame.targetname:SetEffectGlow({strength=10, blurX=2, blurY=2})
	newFrame.targetname:SetText("")
	newFrame.targetname:SetPoint("TOPLEFT", newFrame.buttonBG[n], "BOTTOMLEFT", 0, 0)
	
	--return the new frame.
	return newFrame
end

--returns the value for what the height of a single frame should be
function GW.makeHeight()
	local height = GW.settings.health + GW.settings.mana + GW.settings.target + (GW.settings.button * GW.settings.buttons) + (GW.settings.timer * GW.settings.timers) + (2*GW.settings.padding)
	return height
end

--sets the visibility of all frames.
function GW.setAllVisible(visible)
	for i=1,20 do
		GW.UI.group[i]:SetEnabled(visible)
	end
end

--runs every frame
function GW.onUpdate(event)
	--if everything is loaded
	if GW.enableUpdate == true then
		--only from every {GW.framesPerUpdate} frames
		--this prevents drops in framerate
		GW.updateCounter = GW.updateCounter + 1
		if GW.updateCounter >= GW.framesPerUpdate then
			GW.updateCounter = 0
			
			--update the timers with the latest time passed
			if (Inspect.Time.Frame() - GW.lastUpdate) > 0 then
				GW.updateTimers(Inspect.Time.Frame() - GW.lastUpdate)
				GW.lastUpdate = Inspect.Time.Frame()
			end
		end
	elseif GW.tryingToLoad == false then
		--addon not loaded yet
		--make sure ability list is available
		if Inspect.Ability.New.List() and Inspect.Buff.List("player") and gwSpecs then
			--load addon
			GW.loadAddon()
			GW.enableUpdate = true
		end
	end
	
	--if the option frame is showing
	if GW.UI.OPT.frame and GW.UI.OPT.frame:GetVisible() then
		if GW.settings.visible then
			--make all frames visible, to allow easier editing of sizes
			for i=1,20 do
				GW.UI.group[i].frame:SetAlpha(1)
				GW.UI.group[i]:SetEnabled(true)
			end
			GW.UI.group[21]:SetEnabled(false)
			GW.UI.group[22]:SetEnabled(false)
		end
		if GW.settings.petVisible then
			GW.UI.group[22]:SetEnabled(true)
		end
	end
end

--runs when variables are loaded
function GW.onLoadVars(event, addonid)
	if addonid == "GroupWatch" then
		if gwSpecs == nil then
			gwSpecs = {}
		end
	end
end

function GW.onSaveVars(event, addonid)
	if addonid == "GroupWatch" then
		gwSpecs[GW.currentRole].buttons = GW.buttons
		gwSpecs[GW.currentRole].settings = GW.settings
	end
end

function GW.onLoadAddon(hEvent)
	--see onUpdate for loading
end

--runs when the player changes his role
function GW.onRoleChange(event, role)
	--make sure everything is loaded
	if GW.enableUpdate then
		--swap out the specs
		gwSpecs[GW.currentRole].buttons = GW.buttons
		gwSpecs[GW.currentRole].settings = GW.settings
		GW.currentRole = role
		if gwSpecs[GW.currentRole] == nil then
			gwSpecs[GW.currentRole] = {}
		end
		GW.buttons = gwSpecs[GW.currentRole].buttons
		GW.settings = gwSpecs[GW.currentRole].settings
		--fix up the data
		GW.countFixData = 1
		GW.fixData()
		GW.fixSettings()
		--fix up the options window if shown
		if GW.UI.OPT.frame then
			if GW.UI.OPT.frame:GetVisible() then
				GW.finishEditing()
				GW.startEditing()
			end
		end
		if GW.UI.SET.frame then
			if GW.UI.SET.frame:GetVisible() then
				GW.UI.SET.frame:SetVisible(false)
			end
		end
	end
end

--runs when the /groupwatch slash command is typed
function GW.slashCommand(event, params)
	--if not secure, show settings
	if not Inspect.System.Secure() then
		GW.UI.handle:SetVisible(true)
		GW.UI.petHandle:SetVisible(true)
		GW.UI.OPT.frame:SetVisible(true)
		GW.startEditing()
	end
end

--runs when secure mode is entered
function GW.onEnterSecure()
	--hide the settings and stop any frame movement.
	--this prevents editing of the sizes and positions of restricted frames during secure mode.
	GW.UI.OPT.frame:SetVisible(false)
	GW.UI.SET.frame:SetVisible(false)
	GW.UI.SET.macro:SetKeyFocus(false)
	GW.UI.SET.buff:SetKeyFocus(false)
	if GW.UI.handle.MouseDown then
		local mouseData = Inspect.Mouse()
		GW.settings.handleX = GW.UI.handle:GetLeft()
		GW.settings.handleY = GW.UI.handle:GetTop()
		GW.UI.handle.MouseDown = false
	end
	if GW.UI.petHandle.MouseDown then
		local mouseData = Inspect.Mouse()
		GW.settings.petX = GW.UI.petHandle:GetLeft()
		GW.settings.petY = GW.UI.petHandle:GetTop()
		GW.UI.petHandle.MouseDown = false
	end
	GW.UI.handle:SetVisible(false)
	GW.UI.petHandle:SetVisible(false)
end

--runs when secure mode ends
function GW.onLeaveSecure()
	if GW.countFixData > 0 then
		--role was changed just before entering combat
		--need to update buttons with new abilities
		if not Inspect.System.Secure() then
			Command.System.Watchdog.Quiet()
			GW.fixButtons()
			GW.countFixData = 0
		end
	end
	--update the group, in case anything changed while in secure mode
	GW.updateGroup()
end

--loads the addon.
function GW.loadAddon()
	--record that the loading process has started
	GW.tryingToLoad = true
	--quiet the watchdog
	Command.System.Watchdog.Quiet()
	
	--load settings of curret role
	GW.currentRole = Inspect.TEMPORARY.Role()
	if gwSpecs[GW.currentRole] == nil then
		gwSpecs[GW.currentRole] = {}
	end
	GW.buttons = gwSpecs[GW.currentRole].buttons
	GW.settings = gwSpecs[GW.currentRole].settings
	GW.fixData()
	
	--initialize the ability list
	GW.initAbilities()
	
	--build the user interfaces
	GW.buildUI()
	GW.buildOpt()
	GW.buildSet()
	
	--record the player's cooridnates and ID
	local player = Inspect.Unit.Detail("player")
	GW.playerCoords.x = player.coordX
	GW.playerCoords.y = player.coordY
	GW.playerCoords.z = player.coordZ
	GW.playerID = Inspect.Unit.Lookup("player")
	
	--fix up the frames to look right
	Command.System.Watchdog.Quiet()
	GW.fixSettings()
	--fix up the buttons to work right
	Command.System.Watchdog.Quiet()
	GW.fixButtons()
	
	--record the time and start updating
	GW.lastUpdate = Inspect.Time.Frame()
	GW.enableUpdate = true
	
	--update the information on current party members
	Command.System.Watchdog.Quiet()
	GW.updateGroup()
end

--creates an empty macro table. Currently, only macro.left.default is used.
function GW.newMacroTable()
	local macro = {}
	macro.left = {}
	macro.left.default = { action="cast", name="", target="@unit", id=nil}
	macro.left.shift = { action="cast", name="", target="@unit", id=nil}
	macro.left.ctrl = { action="cast", name="", target="@unit", id=nil}
	macro.left.alt = { action="cast", name="", target="@unit", id=nil}
	
	macro.right = {}
	macro.right.default = { action="cast", name="", target="@unit", id=nil}
	macro.right.shift = { action="cast", name="", target="@unit", id=nil}
	macro.right.ctrl = { action="cast", name="", target="@unit", id=nil}
	macro.right.alt = { action="cast", name="", target="@unit", id=nil}
	
	return macro
end

--creates a usable macro from a macro table.
--macroTable - a table created by calling GW.newMacroTable()
function GW.makeMacro(macroTable)
	local macroText = "#show "..macroTable.default.name.."\n"
	--macroText = macroText .. "stopcasting\n";
	macroText = macroText .. "suppressmacrofailures\n"
	
	macroText = macroText .. macroTable.shift.action .. " [shift] " .. macroTable.shift.target .. " " .. macroTable.shift.name .. "\n"
	macroText = macroText .. macroTable.ctrl.action .. " [ctrl] " .. macroTable.ctrl.target .. " " .. macroTable.ctrl.name .. "\n"
	macroText = macroText .. macroTable.alt.action .. " [alt] " .. macroTable.alt.target .. " " .. macroTable.alt.name .. "\n"
	macroText = macroText .. macroTable.default.action .. " " .. macroTable.default.target .. " " .. macroTable.default.name .. "\n"
	
	return macroText
end

--fixes up a macro.
--macroText - the string containing the macro
--unit - the unit specifier to target with this macro
function GW.fixMacro(macroText, unit)
	--supress failures
	--macroText = "suppressmacrofailures\n"..macroText
	--fix targets. You can use @unit to target the unit, @unitt to target the unit's target, and @unittt to target the unit's target's target.
	macroText = string.gsub(macroText, "@unittt", "@"..unit..".target.target")
	macroText = string.gsub(macroText, "@unitt", "@"..unit..".target")
	macroText = string.gsub(macroText, "@unit", "@"..unit)
	return macroText
end

--this will make sure all settings exist if they don't already in GW.buttons and GW.settings
function GW.fixData()
	--make sure settings exist
	if GW.settings == nil then
		GW.settings = {}
	end
	--override any unset settings with the defaults
	GW.mergeTable(GW.settings, default_settings)
	--make sure buttons exist
	if GW.buttons == nil then
		GW.buttons = {}
	end
	if GW.buttons.timers == nil then
		GW.buttons.timers = {}
	end
	if GW.buttons.buttons == nil then
		GW.buttons.buttons = {}
	end
	for n=1,GW.maxTimers do
		--make sure timer data exists
		if GW.buttons.timers[n] == nil then
			GW.buttons.timers[n] = GW.newMacroTable()
		end
	end
	for n=1,GW.maxButtons do
		--make sure button data exists
		if GW.buttons.buttons[n] == nil then
			GW.buttons.buttons[n] = GW.newMacroTable()
		end
	end
end

--this will make sure all frames match the settings
function GW.fixSettings()
	--fix frames
	GW.setHealthHeight(GW.settings.health)
	GW.setManaHeight(GW.settings.mana)
	GW.setTimerHeight(GW.settings.timer)
	GW.setTimerCount(GW.settings.timers)
	GW.setButtonSize(GW.settings.button)
	GW.setButtonCount(GW.settings.buttons)
	GW.setTargetHeight(GW.settings.target)
	GW.setFrameWidth(GW.settings.width)
	--fix options
	GW.silent = true
	GW.UI.OPT.width:SetPosition(GW.settings.width, true)
	GW.UI.OPT.health:SetPosition(GW.settings.health, true)
	GW.UI.OPT.mana:SetPosition(GW.settings.mana, true)
	GW.UI.OPT.target:SetPosition(GW.settings.target, true)
	GW.UI.OPT.timer:SetPosition(GW.settings.timer, true)
	GW.UI.OPT.timers:SetPosition(GW.settings.timers, true)
	GW.UI.OPT.button:SetPosition(GW.settings.button, true)
	GW.UI.OPT.buttons:SetPosition(GW.settings.buttons, true)
	GW.UI.OPT.growthX:SetSelectedValue(GW.settings.growthX, true)
	GW.UI.OPT.growthY:SetSelectedValue(GW.settings.growthY, true)
	GW.UI.OPT.growthDir:SetSelectedValue(GW.settings.growthDir, true)
	GW.UI.OPT.layout:SetSelectedValue(GW.settings.layout, true)
	GW.UI.OPT.showPlayer:SetChecked(GW.settings.showPlayer)
	GW.UI.OPT.showPet:SetChecked(GW.settings.showPet)
	GW.UI.OPT.visible:SetChecked(GW.settings.visible)
	GW.UI.OPT.petVisible:SetChecked(GW.settings.petVisible)
	GW.silent = false
	--update positions
	GW.updateFramePositions()
	--fix visible
	GW.updateGroup()
	--fix handle
	GW.UI.handle:SetPoint("TOPLEFT", UIParent, "TOPLEFT", GW.settings.handleX, GW.settings.handleY)
	GW.UI.petHandle:SetPoint("TOPLEFT", UIParent, "TOPLEFT", GW.settings.petX, GW.settings.petY)
end

--this will make sure all the macros, icons, etc. match the data
--performance warning
function GW.fixButtons()
	--make sure button data exists
	--fix up timers
	for n=1,GW.maxTimers do
		--make sure timer data exists
		if GW.buttons.timers[n] == nil then
			GW.buttons.timers[n] = GW.newMacroTable()
		end
		--fix macros on each frames
		for i=1,22 do
			local unit = GW.spec[i]
			if GW.buttons.timers[n].left.raw ~= nil then
				GW.UI.group[i].timer[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.buttons.timers[n].left.raw,unit))
			else
				GW.UI.group[i].timer[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.makeMacro(GW.buttons.timers[n].left),unit))
			end
			if GW.buttons.timers[n].right.raw ~= nil then
				GW.UI.group[i].timer[n]:EventMacroSet(Event.UI.Input.Mouse.Right.Down, GW.fixMacro(GW.buttons.timers[n].right.raw,unit))
			else
				GW.UI.group[i].timer[n]:EventMacroSet(Event.UI.Input.Mouse.Right.Down, GW.fixMacro(GW.makeMacro(GW.buttons.timers[n].right),unit))
			end
		end
	end
	--fix up buttons
	for n=1,GW.maxButtons do
		--make sure button data exists
		if GW.buttons.buttons[n] == nil then
			GW.buttons.buttons[n] = GW.newMacroTable()
		end
		--fix macros and icons on each frame
		local id = GW.getShow(GW.buttons.buttons[n].left)
		local icon = nil
		GW.cleaseButtons[n] = nil
		if id ~= nil and not GW.buttons.buttons[n].left.buff then
			--determine if debuffs must be tracked
			if GW.buttons.buttons[n].left.fade == "debuffed" then
				GW.cleaseButtons[n] = true
			end
			--and figure out the icon
			if GW.abilityList[id] ~= nil then
				--ability icon
				icon = GW.abilityList[id].icon
			elseif string.byte(id) == 65 then --A
				local ability = Inspect.Ability.New.Detail(id)
				if ability ~= nil then
					--ability icon
					icon = ability.icon
				end
			elseif string.byte(id) == 105 then --I
				local item = Inspect.Item.Detail(id)
				if item ~= nil then
					--item icon
					icon = item.icon
				end
			end
		end
		for i=1,22 do
			local unit = GW.spec[i]
			--fix macros
			if GW.buttons.buttons[n].left.raw ~= nil then
				GW.UI.group[i].button[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.buttons.buttons[n].left.raw,unit))
			else
				GW.UI.group[i].button[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.makeMacro(GW.buttons.buttons[n].left),unit))
			end
			if GW.buttons.buttons[n].right.raw ~= nil then
				GW.UI.group[i].button[n]:EventMacroSet(Event.UI.Input.Mouse.Right.Down, GW.fixMacro(GW.buttons.buttons[n].right.raw,unit))
			else
				GW.UI.group[i].button[n]:EventMacroSet(Event.UI.Input.Mouse.Right.Down, GW.fixMacro(GW.makeMacro(GW.buttons.buttons[n].right),unit))
			end
			--fix icons
			if icon ~= nil then
				--button has been set
				GW.UI.group[i].buttonBG[n]:SetTexture("Rift", icon)
			else
				GW.UI.group[i].buttonBG[n]:SetTexture(addon.identifier, "img/buttonbg.png")
			end
		end
		--also fix icons on the test frame
		if icon ~= nil then
			--button has been set...
			GW.UI.OPT.testFrame.buttonBG[n]:SetTexture("Rift", icon)
		else
			GW.UI.OPT.testFrame.buttonBG[n]:SetTexture(addon.identifier, "img/buttonbg.png")
		end
		--fade out if unusable
		GW.initFade(n)
	end
end

--merges a table. Any values that do not exist in table n but do exist in table o will be added to o.
--o - the original table. This will be updated with new values, but existing values will not change.
--n - the new table. This will not change, but will be added to table o.
function GW.mergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}
			end
	 	 	if type(o[k]) == 'table' then
	 			GW.mergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function GW.tableSize(t)
	if t == nil then return 0 end
	local count = 0
	for _ in pairs(t) do count = count + 1 end
	return count
end

--------------------------------------------------------------------------------------------
-- Attach events for this script file. -----------------------------------------------------
--------------------------------------------------------------------------------------------
Command.Event.Attach(Event.System.Update.Begin, GW.onUpdate, "onUpdate")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, GW.onLoadVars, "onLoadVars")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, GW.onSaveVars, "onSaveVars")
Command.Event.Attach(Event.Addon.Startup.End, GW.onLoadAddon, "onLoadAddon")
Command.Event.Attach(Event.TEMPORARY.Role, GW.onRoleChange, "onRoleChange")
Command.Event.Attach(Command.Slash.Register("groupwatch"), GW.slashCommand, "SlashCommands")
Command.Event.Attach(Event.System.Secure.Enter, GW.onEnterSecure, "onEnterSecure")
Command.Event.Attach(Event.System.Secure.Leave, GW.onLeaveSecure, "onLeaveSecure")