local addon, GW = ...

--setTimer: true if setting up a macro for a timer, false if button
local setTimer = false
--setRight: true if setting up a right-click macro, false if left.
local setRight = false
--curSet: n of the timer/button currently being set
local curSet = 0
--macroTable: the table containing the macro data currently being edited
local macroTable = nil
--modTable: a subtable of macroTable consisting of the currently edited modifier
local modTable = macroTable

--constants to make the code look better
local TIMER = true
local BUTTON = false
local RIGHT = true
local LEFT = false

GW.silent = false

--begins setting a button.
--timer -- true if we're setting up a timer, false if button
--right -- true if we're setting up a right-click, false if left
--n -- the 'n' of the timer or button
function GW.startSet(timer, right, n)
	--record the settingss
	setTimer = timer
	setRight = right
	curSet = n
	
	--discern which macro table to use
	if setTimer then
		macroTable = GW.buttons.timers[curSet]
	else
		macroTable = GW.buttons.buttons[curSet]
	end
	if setRight then
		macroTable = macroTable.right
	else
		macroTable = macroTable.left
	end
	if (not (macroTable.raw == nil)) then
		--raw macro
		GW.silent = true
		GW.UI.SET.custom:SetChecked(true)
		GW.UI.SET.macro:SetVisible(true)
		GW.UI.SET.area:SetVisible(false)
		GW.UI.SET.modifierText:SetVisible(false)
		GW.UI.SET.modifier:SetVisible(false)
		GW.UI.SET.macro:SetText(macroTable.raw)
		GW.silent = false
	else
		GW.silent = true
		GW.UI.SET.custom:SetChecked(false)
		GW.UI.SET.macro:SetVisible(false)
		GW.UI.SET.area:SetVisible(true)
		GW.UI.SET.modifierText:SetVisible(true)
		GW.UI.SET.modifier:SetVisible(true)
		GW.silent = false
	end
	if macroTable.buff then
		GW.UI.SET.buff:SetText(macroTable.buff)
	else
		GW.UI.SET.buff:SetText("")
	end
	if setTimer then
		--disable dim when buffed
		GW.UI.SET.fadeOut:SetSelectedIndex(1, true)
		GW.UI.SET.fadeOut:SetEnabled(false)
	else
		--enable dim when buffed
		GW.UI.SET.fadeOut:SetEnabled(true)
		if macroTable.fade then
			GW.UI.SET.fadeOut:SetSelectedValue(macroTable.fade, true)
		else
			GW.UI.SET.fadeOut:SetSelectedIndex(1, true)
		end
	end
	
	--start by editing the ability with no modifier
	modTable = macroTable.default
	GW.UI.SET.modifier:SetSelectedValue("default", true)
	
	--make the set frame visible
	GW.UI.SET.frame:SetVisible(true)
	GW.updateSetTable()
end

--saves the current settings from the set window
--currently this functions as an auto-save
function GW.saveBasic()
	GW.saveSet(macroTable.default.id, GW.makeMacro(macroTable))
end

function GW.saveRaw()
	GW.saveSet(GW.getShow(macroTable), macroTable.raw)
end

function GW.saveSet(id, macro)
	if not Inspect.System.Secure() then
		--figure out what event we're using, left or rights
		local event = Event.UI.Input.Mouse.Left.Down
		if setRight then --setting a right-click event
			event = Event.UI.Input.Mouse.Right.Down
		end
		--find correct icon
		local icon = nil
		GW.cleaseButtons[curSet] = nil
		if id ~= nil and not setRight then
			if not GW.buttons.buttons[curSet].left.buff then
				--determine if debuffs must be tracked
				if GW.buttons.buttons[curSet].left.fade == "debuffed" then
					GW.cleaseButtons[curSet] = true
				end
				--and figure out the icon
				if GW.abilityList[id] ~= nil then
					--ability icon
					icon = GW.abilityList[id].icon
				elseif string.byte(id) == 65 then --A
					local ability = Inspect.Ability.New.Detail(id)
					if ability ~= nil then
						--ability icon
						icon = ability.icon
					end
				elseif string.byte(id) == 105 then --I
					local item = Inspect.Item.Detail(id)
					if item ~= nil then
						--item icon
						icon = item.icon
					end
				end
			end
		end
		--for each frame
		for i=1,22 do
			--make a unit specifier
			local unit = GW.spec[i]
			--set the macro
			if setTimer then --setting a timer
				GW.UI.group[i].timer[curSet]:EventMacroSet(event, GW.fixMacro(macro,unit))
			else --setting a button
				GW.UI.group[i].button[curSet]:EventMacroSet(event, GW.fixMacro(macro,unit))
				--and set the icon
				if icon ~= nil then
					GW.UI.group[i].buttonBG[curSet]:SetTexture("Rift", icon)
				else
					GW.UI.group[i].buttonBG[curSet]:SetTexture(addon.identifier, "img/buttonbg.png")
				end
			end
		end
		--also set the icon for the test frame
		if not setTimer then --only set icons for buttons on left-click
			if icon ~= nil then
				GW.UI.OPT.testFrame.buttonBG[curSet]:SetTexture("Rift", icon)
			else
				GW.UI.OPT.testFrame.buttonBG[curSet]:SetTexture(addon.identifier, "img/buttonbg.png")
			end
		end
	end
end

function GW.getShow(macroTable)
	if macroTable.raw == nil then return macroTable.default.id end
	local showName = GW.getShowName(macroTable)
	if GW.abilityIDs[showName] ~= nil then
		return GW.abilityIDs[showName]
	end
	return nil
end

function GW.getShowName(macroTable)
	if macroTable.raw == nil then return macroTable.default.name end
	local iStart = string.find(macroTable.raw, "#show ")
	local iEnd = string.find(macroTable.raw, "\n")
	if iStart ~= nil and iEnd ~= nil then
		local show = string.sub(macroTable.raw, (iStart+6), (iEnd-1))
		return show
	end
	return nil
end

--creates the set interface
function GW.buildSet()
	--context is currently unused, we use GW.optionContext here
	GW.setContext = UI.CreateContext(addon.identifier)
	
	--the set frame, modeled after a RiftWindow
	GW.UI.SET.frame = UI.CreateFrame("RiftWindow", "GW.UI.SET.frame", GW.optionContext)
	GW.UI.SET.frame:SetWidth(300)
	GW.UI.SET.frame:SetHeight(360)
	GW.UI.SET.frame:SetLayer(2)
	GW.UI.SET.frame:SetTitle("Set Ability")
	GW.UI.SET.frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	GW.UI.SET.frame:SetVisible(false)
	
	--close button, closes the set frame (changes already saved)
	GW.UI.SET.close = UI.CreateFrame("RiftButton", "GW.UI.SET.close", GW.UI.SET.frame)
	GW.UI.SET.close:SetSkin("close")
	GW.UI.SET.close:SetPoint("TOPRIGHT", GW.UI.SET.frame, "TOPRIGHT", -8, 16)
	GW.UI.SET.close:SetLayer(3)
	GW.UI.SET.close:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		GW.UI.SET.frame:SetVisible(false)
		GW.UI.SET.macro:SetKeyFocus(false)
		GW.UI.SET.buff:SetKeyFocus(false)
	end, "Event.UI.Input.Mouse.Left.Click")
	
	--option to edit the macro raw
	GW.UI.SET.custom = UI.CreateFrame("SimpleCheckbox", "GW.UI.SET.custom", GW.UI.SET.frame)
	GW.UI.SET.custom:SetWidth(100)
	GW.UI.SET.custom:SetHeight(50)
	GW.UI.SET.custom:SetFontSize(12)
	GW.UI.SET.custom:SetLayer(10)
	GW.UI.SET.custom:SetText("Edit Raw Macro")
	GW.UI.SET.custom:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 50)
	function GW.UI.SET.custom.Event:CheckboxChange()
		if GW.silent == true then
			return
		end
		if GW.UI.SET.custom:GetChecked() then
			macroTable.raw = GW.makeMacro(macroTable)
			GW.UI.SET.macro:SetText(macroTable.raw)
			GW.UI.SET.macro:SetVisible(true)
			GW.UI.SET.area:SetVisible(false)
			GW.UI.SET.modifierText:SetVisible(false)
			GW.UI.SET.modifier:SetVisible(false)
		else
			GW.UI.SET.comfirm:SetVisible(true)
		end
	end
	
	--option to fade out when already buffed
	GW.UI.SET.fadeOut = UI.CreateFrame("SimpleSelect", "GW.UI.SET.fadeOut", GW.UI.SET.frame)
	GW.UI.SET.fadeOut:SetHeight(50)
	GW.UI.SET.fadeOut:SetFontSize(12)
	GW.UI.SET.fadeOut:SetLayer(10)
	GW.UI.SET.fadeOut:SetItems({"Fade Out: Never","When buffed","When cleansed","When unusable"}, {"never","buffed","debuffed","usable"})
	GW.UI.SET.fadeOut:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 150, 50)
	function GW.UI.SET.fadeOut.Event:ItemSelect(item, value, index)
		if value == "never" then
			macroTable.fade = nil
		else
			macroTable.fade = value
		end
		GW.initFade(curSet)
	end
	GW.UI.SET.fadeOut:ResizeToFit()
	
	--label for changing which modifier we're currently editing
	GW.UI.SET.modifierText = UI.CreateFrame("Text", "GW.UI.SET.modifierText", GW.UI.SET.frame)
	GW.UI.SET.modifierText:SetWidth(250)
	GW.UI.SET.modifierText:SetHeight(25)
	GW.UI.SET.modifierText:SetFontSize(12)
	GW.UI.SET.modifierText:SetFontColor(1,1,0)
	GW.UI.SET.modifierText:SetEffectGlow({strength=10, blurX=2, blurY=2})
	GW.UI.SET.modifierText:SetText("Select the modifier you would like to edit:")
	GW.UI.SET.modifierText:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 75)
	--slider for changing which modifier we're currently editing
	GW.UI.SET.modifier = UI.CreateFrame("SimpleSelect", "GW.UI.SET.modifier", GW.UI.SET.frame)
	GW.UI.SET.modifier:SetWidth(250)
	GW.UI.SET.modifier:SetItems({"None","Shift","Ctrl","Alt"}, {"default","shift","ctrl","alt"})
	GW.UI.SET.modifier:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 100)
	function GW.UI.SET.modifier.Event:ItemSelect(item, value, index)
		modTable = macroTable[value]
		--update the rest of the interface to edit based on this modifier
		GW.updateSetTable()
	end
	
	--area in which the spell bound to the selected modifier is edited
	GW.UI.SET.area = UI.CreateFrame("Texture", "GW.UI.SET.area", GW.UI.SET.frame)
	GW.UI.SET.area:SetLayer(1)
	GW.UI.SET.area:SetWidth(250)
	GW.UI.SET.area:SetHeight(125)
	GW.UI.SET.area:SetTexture(addon.identifier, "img/frameClear.png")
	GW.UI.SET.area:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 125)
	
	--the box to which the user must drag a spell or item to bind it
	GW.UI.SET.buttonBG = UI.CreateFrame("Texture", "GW.UI.SET.buttonBG", GW.UI.SET.area)
	GW.UI.SET.buttonBG:SetWidth(50)
	GW.UI.SET.buttonBG:SetHeight(50)
	GW.UI.SET.buttonBG:SetLayer(10)
	GW.UI.SET.buttonBG:SetTexture(addon.identifier, "img/buttonbg.png")
	GW.UI.SET.buttonBG:SetPoint("TOPLEFT", GW.UI.SET.area, "TOPLEFT", 25, 20)
	-- set the tooltip handlers
	GW.UI.SET.buttonBG:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self)
		if modTable.id ~= nil then
			GW.tooltipID = modTable.id
			Command.Tooltip(modTable.id)
			GW.UI.SET.buttonHover:SetTexture(addon.identifier, "img/buttonhover.png")
		else
			local ctype, held = Inspect.Cursor()
			if ctype ~= nil then
				GW.UI.SET.buttonHover:SetTexture(addon.identifier, "img/buttonhover.png")
			end
		end
	end, "Event.UI.Input.Mouse.Cursor.In")
	GW.UI.SET.buttonBG:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self)
		if modTable.id ~= nil then
			if GW.tooltipID == modTable.id then
				Command.Tooltip(nil)
			end
		end
		GW.UI.SET.buttonHover:SetTexture(addon.identifier, "img/nothing.png")
	end, "Event.UI.Input.Mouse.Cursor.Out")
	-- allow dropping of abilities onto this frame
	GW.UI.SET.buttonBG:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, event)
		if not Inspect.System.Secure() then
			local ctype, held = Inspect.Cursor()
			if ctype == "ability" and held then
				local ability = GW.abilityList[held]
				if ability == nil then ability = Inspect.Ability.New.Detail(held) end
				if ability ~= nil and ability.name then
					modTable.action="cast"
					modTable.name = ability.name
					modTable.id = held
					self:SetTexture("Rift", ability.icon)
					Command.Cursor(nil)
					--if editing the default modifier and the ability is a cleasing ability
					if modTable == macroTable.default and GW.cleanser[held] then
						--ability is cleasing, auto set the fade to cleanse
						GW.UI.SET.fadeOut:SetSelectedValue("debuffed")
					end
				end
			elseif ctype == "item" and held then
				local item = Inspect.Item.Detail(held)
				if item ~= nil and item.name then
					modTable.action="use"
					modTable.name = item.name
					modTable.id = held
					self:SetTexture("Rift", item.icon)
					Command.Cursor(nil)
				end
			end
			GW.saveBasic()
		end
	end, "Event.UI.Input.Mouse.Left.Up")
	-- allow picking up of abilities from this frame
	GW.UI.SET.buttonBG:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, event)
		if not Inspect.System.Secure() then
			if modTable.id ~= nil then
				if GW.tooltipID == modTable.id then
					Command.Tooltip(nil)
				end
				Command.Cursor(modTable.id)
				--if editing the default modifier and the ability is a cleasing ability
				if modTable == macroTable.default and GW.cleanser[modTable.id] then
					--ability is cleasing, auto set the fade to never
					if GW.UI.SET.fadeOut:GetSelectedValue() == "debuffed" then
						GW.UI.SET.fadeOut:SetSelectedValue("never")
					end
				end
			end
			modTable.action="cast"
			modTable.name = ""
			modTable.id = nil
			self:SetTexture(addon.identifier, "img/buttonbg.png")
			GW.saveBasic()
		end
	end, "Event.UI.Input.Mouse.Left.Down")
	-- right-click to clear the current binding.
	GW.UI.SET.buttonBG:EventAttach(Event.UI.Input.Mouse.Right.Up, function(self, event)
		if not Inspect.System.Secure() then
			if modTable.id ~= nil then
				if GW.tooltipID == modTable.id then
					Command.Tooltip(nil)
				end
				--if editing the default modifier and the ability is a cleasing ability
				if modTable == macroTable.default and GW.cleanser[modTable.id] then
					--ability is cleasing, auto set the fade to never
					if GW.UI.SET.fadeOut:GetSelectedValue() == "debuffed" then
						GW.UI.SET.fadeOut:SetSelectedValue("never")
					end
				end
			end
			modTable.action="cast"
			modTable.name = ""
			modTable.id = nil
			self:SetTexture(addon.identifier, "img/buttonbg.png")
			GW.saveBasic()
		end
	end, "Event.UI.Input.Mouse.Right.Up")
	
	--a hover effect when the user hovers over this button
	GW.UI.SET.buttonHover = UI.CreateFrame("Texture", "GW.UI.SET.buttonHover", GW.UI.SET.area)
	GW.UI.SET.buttonHover:SetWidth(50)
	GW.UI.SET.buttonHover:SetHeight(50)
	GW.UI.SET.buttonHover:SetLayer(15)
	GW.UI.SET.buttonHover:SetTexture(addon.identifier, "img/nothing.png")
	GW.UI.SET.buttonHover:SetPoint("TOPLEFT", GW.UI.SET.area, "TOPLEFT", 25, 20)
	
	--a border to make the box look more like an action button
	GW.UI.SET.buttonBorder = UI.CreateFrame("Texture", "GW.UI.SET.buttonBorder", GW.UI.SET.area)
	GW.UI.SET.buttonBorder:SetWidth(50)
	GW.UI.SET.buttonBorder:SetHeight(50)
	GW.UI.SET.buttonBorder:SetLayer(20)
	GW.UI.SET.buttonBorder:SetTexture(addon.identifier, "img/buttonbig.png")
	GW.UI.SET.buttonBorder:SetPoint("TOPLEFT", GW.UI.SET.area, "TOPLEFT", 25, 20)
	
	--text explaining what to do
	GW.UI.SET.text = UI.CreateFrame("Text", "GW.UI.SET.text", GW.UI.SET.area)
	GW.UI.SET.text:SetWidth(150)
	GW.UI.SET.text:SetHeight(60)
	GW.UI.SET.text:SetFontSize(12)
	GW.UI.SET.text:SetLayer(10)
	GW.UI.SET.text:SetFontColor(1,1,0)
	GW.UI.SET.text:SetEffectGlow({strength=10, blurX=2, blurY=2})
	GW.UI.SET.text:SetText("Drag an ability or item\ninto the box to bind it.\nThen, set it's target.")
	GW.UI.SET.text:SetPoint("TOPLEFT", GW.UI.SET.area, "TOPLEFT", 100, 15)
	
	--label for changing the desired target
	GW.UI.SET.targetText = UI.CreateFrame("Text", "GW.UI.SET.targetText", GW.UI.SET.area)
	GW.UI.SET.targetText:SetWidth(200)
	GW.UI.SET.targetText:SetHeight(25)
	GW.UI.SET.targetText:SetFontSize(12)
	GW.UI.SET.targetText:SetLayer(10)
	GW.UI.SET.targetText:SetFontColor(1,1,0)
	GW.UI.SET.targetText:SetEffectGlow({strength=10, blurX=2, blurY=2})
	GW.UI.SET.targetText:SetText("Target: unit")
	GW.UI.SET.targetText:SetPoint("TOPLEFT", GW.UI.SET.area, "TOPLEFT", 25,75)
	--slider for changing the desired target
	GW.UI.SET.target = UI.CreateFrame("RiftSlider", "GW.UI.SET.timers", GW.UI.SET.area)
	GW.UI.SET.target:SetWidth(200)
	GW.UI.SET.target:SetHeight(25)
	GW.UI.SET.target:SetLayer(10)
	GW.UI.SET.target:SetRange(0, 3)
	GW.UI.SET.target:SetPosition(0)
	GW.UI.SET.target:SetPoint("TOPLEFT", GW.UI.SET.area, "TOPLEFT", 25, 100)
	GW.UI.SET.target:EventAttach(Event.UI.Slider.Change, function(self, h)
		if self:GetPosition() == 0 then
			GW.UI.SET.targetText:SetText("Target: unit")
			modTable.target = "@unit"
		elseif self:GetPosition() == 1 then
			GW.UI.SET.targetText:SetText("Target: unit-target")
			modTable.target = "@unitt"
		elseif self:GetPosition() == 2 then
			GW.UI.SET.targetText:SetText("Target: unit-target-target")
			modTable.target = "@unittt"
		else
			GW.UI.SET.targetText:SetText("Target: unit-pet")
			modTable.target = "@unit.pet"
		end
		GW.saveBasic()
	end, "Event.UI.Slider.Change")
	
	--the custom macro input dialog
	GW.UI.SET.macro = UI.CreateFrame("SimpleTextArea", "GW.UI.SET.macro", GW.UI.SET.frame)
	GW.UI.SET.macro:SetWidth(250)
	GW.UI.SET.macro:SetHeight(175)
	GW.UI.SET.macro:SetLayer(90)
	GW.UI.SET.macro:SetBorder(2, 1, 1, 1, 1)
	GW.UI.SET.macro:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 75)
	GW.UI.SET.macro:SetVisible(false)
	function GW.UI.SET.macro.Event:TextAreaChange(item)
		macroTable.raw = string.gsub(GW.UI.SET.macro:GetText(), "\13", "\n")
		GW.saveRaw()
	end
	
	--------------------------------------------------
	
	GW.UI.SET.buffText = UI.CreateFrame("Text", "GW.UI.SET.buffText", GW.UI.SET.frame)
	GW.UI.SET.buffText:SetWidth(260)
	GW.UI.SET.buffText:SetHeight(60)
	GW.UI.SET.buffText:SetFontSize(12)
	GW.UI.SET.buffText:SetLayer(10)
	GW.UI.SET.buffText:SetFontColor(1,1,0)
	GW.UI.SET.buffText:SetEffectGlow({strength=10, blurX=2, blurY=2})
	GW.UI.SET.buffText:SetText("Specify a buff by name to watch for (advanced)\nWill show buff icon and data instead of ability\nLeave blank to show default ability info (default)")
	GW.UI.SET.buffText:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 260)
	
	GW.UI.SET.buff = UI.CreateFrame("RiftTextfield", "GW.UI.SET.buff", GW.UI.SET.frame)
	GW.UI.SET.buff:SetWidth(250)
	GW.UI.SET.buff:SetHeight(20)
	GW.UI.SET.buff:SetLayer(10)
	GW.UI.SET.buff:SetPoint("TOPLEFT", GW.UI.SET.frame, "TOPLEFT", 25, 320)
	GW.UI.SET.buff:EventAttach(Event.UI.Textfield.Change, function(self, h)
		if self:GetText() == "" then
			macroTable.buff = nil
		else
			macroTable.buff = self:GetText()
		end
		if macroTable.raw then
			GW.saveRaw()
		else
			GW.saveBasic()
		end
	end, "Event.UI.Textfield.Change")
	Library.LibSimpleWidgets.SetBorder("plain", GW.UI.SET.buff, 1, 1, 1, 1, 1, "tblr")
	
	--------------------------------------------------
	
	--the comfirmation frame, modeled after a RiftWindow
	GW.UI.SET.comfirm = UI.CreateFrame("RiftWindow", "GW.UI.SET.comfirm", GW.optionContext)
	GW.UI.SET.comfirm:SetWidth(300)
	GW.UI.SET.comfirm:SetHeight(100)
	GW.UI.SET.comfirm:SetLayer(100)
	GW.UI.SET.comfirm:SetTitle("")
	GW.UI.SET.comfirm:SetPoint("CENTER", UIParent, "CENTER", 0, -100)
	GW.UI.SET.comfirm:SetVisible(false)
	
	GW.UI.SET.comfirmText = UI.CreateFrame("Text", "GW.UI.SET.comfirmText", GW.UI.SET.comfirm)
	GW.UI.SET.comfirmText:SetWidth(100)
	GW.UI.SET.comfirmText:SetHeight(25)
	GW.UI.SET.comfirmText:SetFontSize(12)
	GW.UI.SET.comfirmText:SetLayer(10)
	GW.UI.SET.comfirmText:SetFontColor(1,1,0)
	GW.UI.SET.comfirmText:SetEffectGlow({strength=10, blurX=2, blurY=2})
	GW.UI.SET.comfirmText:SetText("Your custom macro will be lost!")
	GW.UI.SET.comfirmText:SetPoint("TOPLEFT", GW.UI.SET.comfirm, "TOPLEFT", 25, 25)
	
	GW.UI.SET.comfirmYes = UI.CreateFrame("RiftButton", "GW.UI.SET.comfirmYes", GW.UI.SET.comfirm)
	GW.UI.SET.comfirmYes:SetWidth(100)
	GW.UI.SET.comfirmYes:SetHeight(25)
	GW.UI.SET.comfirmYes:SetLayer(3)
	GW.UI.SET.comfirmYes:SetText("Ok")
	GW.UI.SET.comfirmYes:SetPoint("TOPLEFT", GW.UI.SET.comfirm, "TOPLEFT", 50, 50)
	GW.UI.SET.comfirmYes:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		macroTable.raw = nil;
		GW.UI.SET.comfirm:SetVisible(false)
		GW.UI.SET.macro:SetVisible(false)
		GW.UI.SET.area:SetVisible(true)
		GW.UI.SET.modifierText:SetVisible(true)
		GW.UI.SET.modifier:SetVisible(true)
		GW.UI.SET.macro:SetKeyFocus(false)
		GW.saveBasic()
	end, "Event.UI.Input.Mouse.Left.Click")
	
	GW.UI.SET.comfirmNo = UI.CreateFrame("RiftButton", "GW.UI.SET.comfirmNo", GW.UI.SET.comfirm)
	GW.UI.SET.comfirmNo:SetWidth(100)
	GW.UI.SET.comfirmNo:SetHeight(25)
	GW.UI.SET.comfirmNo:SetLayer(3)
	GW.UI.SET.comfirmNo:SetText("Cancel")
	GW.UI.SET.comfirmNo:SetPoint("TOPLEFT", GW.UI.SET.comfirm, "TOPLEFT", 150, 50)
	GW.UI.SET.comfirmNo:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		GW.silent = true
		GW.UI.SET.custom:SetChecked(true)
		GW.UI.SET.comfirm:SetVisible(false)
		GW.silent = false
	end, "Event.UI.Input.Mouse.Left.Click")
end

--creates a dummy frame which can be used to bind abilities
--abilities can be bound by dragging the ability onto the dummy frame
--or clicking an ability frame (timer or button) for advanced options
function GW.createTestFrame(x, y)
	local newFrame = {} -- a new frame to return
	
	--main frame. Dummy frame, representing no one.
	newFrame.frame = UI.CreateFrame("Frame", "GW.UI.OPT.testFrame.frame", GW.UI.OPT.frame)
	newFrame.frame:SetWidth(GW.UI.width)
	newFrame.frame:SetHeight(GW.UI.height)
	newFrame.frame:SetLayer(1)
	newFrame.frame:SetPoint("TOPLEFT", GW.UI.OPT.frame, "TOPLEFT", x, y)
	
	--frame background texture.
	newFrame.frameBG = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.frameBG", newFrame.frame)
	newFrame.frameBG:SetLayer(1)
	newFrame.frameBG:SetTexture(addon.identifier, "img/frame.png")
	newFrame.frameBG:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", 0, 0)
	newFrame.frameBG:SetPoint("BOTTOMRIGHT", newFrame.frame, "BOTTOMRIGHT", 0, 0)
	
	--health bar for display only. always full.
	newFrame.healthbar = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.healthbar", newFrame.frame)
	newFrame.healthbar:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.healthbar:SetHeight(GW.settings.health)
	newFrame.healthbar:SetLayer(20)
	newFrame.healthbar:SetTexture(addon.identifier, "img/bar.png")
	newFrame.healthbar:SetBackgroundColor(0, 1, 0)
	newFrame.healthbar:SetPoint("TOPLEFT", newFrame.frame, "TOPLEFT", GW.settings.padding, GW.settings.padding)
	
	--mana bar for display only. always full.
	newFrame.manabar = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.manabar", newFrame.frame)
	newFrame.manabar:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.manabar:SetHeight(GW.settings.mana)
	newFrame.manabar:SetLayer(20)
	newFrame.manabar:SetTexture(addon.identifier, "img/bar.png")
	newFrame.manabar:SetBackgroundColor(0, 0, 1)
	newFrame.manabar:SetPoint("TOPLEFT", newFrame.healthbar, "BOTTOMLEFT", 0, 0)
	
	newFrame.timerbar = {}
	newFrame.timerHover = {}
	for n=1,GW.maxTimers do
		--------------------------------------------------------------------------------------------
		-- timerbar frame. Drag abilities to this bar to set the corresponding timerbars  ----------
		-- for each raid frame, or click this bar for advanced options.  ---------------------------
		--------------------------------------------------------------------------------------------
		newFrame.timerbar[n] = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.timerbar"..n, newFrame.frame)
		newFrame.timerbar[n]:SetWidth(GW.settings.width-(2*GW.settings.padding))
		newFrame.timerbar[n]:SetHeight(GW.settings.timer)
		newFrame.timerbar[n]:SetLayer(20)
		newFrame.timerbar[n]:SetTexture(addon.identifier, "img/barbg.png")
		-- attach it to mana (first timer) or the previous timer
		if n == 1 then
			newFrame.timerbar[n]:SetPoint("TOPLEFT", newFrame.manabar, "BOTTOMLEFT", 0, 0)
		else
			newFrame.timerbar[n]:SetPoint("TOPLEFT", newFrame.timerbar[n-1], "BOTTOMLEFT", 0, 0)
		end
		-- set the tooltip handlers
		newFrame.timerbar[n]:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self)
			if GW.buttons.timers[n] then
				local show = GW.getShow(GW.buttons.timers[n].left)
				if show ~= nil then
					GW.tooltipID = show
					Command.Tooltip(show)
				end
				newFrame.timerHover[n]:SetTexture(addon.identifier, "img/barhover.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.In")
		newFrame.timerbar[n]:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self)
			if GW.buttons.timers[n] then
				if GW.tooltipID == GW.getShow(GW.buttons.timers[n].left) then
					Command.Tooltip(nil)
				end
				newFrame.timerHover[n]:SetTexture(addon.identifier, "img/nothing.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.Out")
		-- click to open up advanced set interface
		newFrame.timerbar[n]:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
			GW.startSet(TIMER, LEFT, n)
		end, "Event.UI.Input.Mouse.Left.Down")
		newFrame.timerbar[n]:EventAttach(Event.UI.Input.Mouse.Right.Down, function(self, h)
			GW.startSet(TIMER, RIGHT, n)
		end, "Event.UI.Input.Mouse.Right.Down")
		-- allow dropping of abilities onto this frame
		newFrame.timerbar[n]:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, event)
			if not Inspect.System.Secure() then
				local ctype, held = Inspect.Cursor()
				if ctype == "ability" and held then
					local ability = GW.abilityList[held]
					if ability == nil then ability = Inspect.Ability.New.Detail(held) end
					if ability ~= nil and ability.name then
						GW.buttons.timers[n].left.default.action="cast"
						GW.buttons.timers[n].left.default.target="@unit"
						GW.buttons.timers[n].left.default.name = ability.name
						GW.buttons.timers[n].left.default.id = held
						for slot=1,22 do
							local unit = GW.spec[slot]
							GW.UI.group[slot].timer[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.makeMacro(GW.buttons.timers[n].left),unit))
						end
						Command.Cursor(nil)
						GW.updateSetTable()
					end
				elseif ctype == "item" and held then
					local item = Inspect.Item.Detail(held)
					if item ~= nil and item.name then
						GW.buttons.timers[n].left.default.action="use"
						GW.buttons.timers[n].left.default.target="@unit"
						GW.buttons.timers[n].left.default.name = item.name
						GW.buttons.timers[n].left.default.id = held
						for slot=1,22 do
							local unit = GW.spec[slot]
							GW.UI.group[slot].timer[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.makeMacro(GW.buttons.timers[n].left),unit))
						end
						Command.Cursor(nil)
						GW.updateSetTable()
					end
				end
			end
		end, "Event.UI.Input.Mouse.Left.Up")
		
		--------------------------------------------------------------------------------------------
		-- timerHover frame. Texture frame which changes width based on time remaining. --------------
		--------------------------------------------------------------------------------------------
		newFrame.timerHover[n] = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.timerHover."..n, newFrame.frame)
		newFrame.timerHover[n]:SetWidth(0)
		newFrame.timerHover[n]:SetHeight(GW.settings.timer)
		newFrame.timerHover[n]:SetLayer(25)
		newFrame.timerHover[n]:SetTexture(addon.identifier, "img/nothing.png")
		-- attach it to mana (first timer) or the previous timer
		if n == 1 then
			newFrame.timerHover[n]:SetPoint("TOPLEFT", newFrame.manabar, "BOTTOMLEFT", 0, 0)
		else
			newFrame.timerHover[n]:SetPoint("TOPLEFT", newFrame.timerbar[n-1], "BOTTOMLEFT", 0, 0)
		end
	end
	
	newFrame.buttonBG = {}
	newFrame.buttonBorder = {}
	newFrame.buttonHover = {}
	for n=1,GW.maxButtons do
		--------------------------------------------------------------------------------------------
		-- buttonBG frame. Drag abilities to this button to set the corresponding buttons  ---------
		-- for each raid frame, or click this button for advanced options.  ------------------------
		--------------------------------------------------------------------------------------------
		newFrame.buttonBG[n] = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.buttonBG"..n, newFrame.frame)
		newFrame.buttonBG[n]:SetWidth(GW.settings.button)
		newFrame.buttonBG[n]:SetHeight(GW.settings.button)
		newFrame.buttonBG[n]:SetLayer(10)
		newFrame.buttonBG[n]:SetMouseMasking("limited")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonBG[n]:SetPoint("TOPLEFT", newFrame.timerbar[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonBG[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonBG[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
		-- set the tooltip handlers
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Cursor.In, function(self)
			if GW.buttons.buttons[n] then
				local show = GW.getShow(GW.buttons.buttons[n].left)
				if show ~= nil then
					GW.tooltipID = show
					Command.Tooltip(show)
				end
				newFrame.buttonHover[n]:SetTexture(addon.identifier, "img/buttonhover.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.In")
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function(self)
			if GW.buttons.buttons[n] then
				if GW.tooltipID == GW.getShow(GW.buttons.buttons[n].left) then
					Command.Tooltip(nil)
				end
				newFrame.buttonHover[n]:SetTexture(addon.identifier, "img/nothing.png")
			end
		end, "Event.UI.Input.Mouse.Cursor.Out")
		-- click to open up advanced set interface
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
			GW.startSet(BUTTON, LEFT, n)
		end, "Event.UI.Input.Mouse.Left.Down")
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Right.Down, function(self, h)
			GW.startSet(BUTTON, RIGHT, n)
		end, "Event.UI.Input.Mouse.Right.Down")
		-- allow dropping of abilities onto this frame
		newFrame.buttonBG[n]:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, event)
			if not Inspect.System.Secure() then
				local ctype, held = Inspect.Cursor()
				if ctype == "ability" and held then
					local ability = GW.abilityList[held]
					if ability == nil then ability = Inspect.Ability.New.Detail(held) end
					if ability ~= nil and ability.name then
						GW.buttons.buttons[n].left.default.action="cast"
						GW.buttons.buttons[n].left.default.target="@unit"
						GW.buttons.buttons[n].left.default.name = ability.name
						GW.buttons.buttons[n].left.default.id = held
						if GW.cleanser[held] then
							--ability is cleasing, auto set the fade to cleanse
							GW.buttons.buttons[n].left.fade = "debuffed"
							GW.UI.SET.fadeOut:SetSelectedValue("debuffed", true)
						end
						for slot=1,22 do
							local unit = GW.spec[slot]
							GW.UI.group[slot].button[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.makeMacro(GW.buttons.buttons[n].left),unit))
							GW.UI.group[slot].buttonBG[n]:SetTexture("Rift", ability.icon)
						end
						self:SetTexture("Rift", ability.icon)
						Command.Cursor(nil)
						GW.updateSetTable()
					end
				elseif ctype == "item" and held then
					local item = Inspect.Item.Detail(held)
					if item ~= nil and item.name then
						GW.buttons.buttons[n].left.default.action="use"
						GW.buttons.buttons[n].left.default.target="@unit"
						GW.buttons.buttons[n].left.default.name = item.name
						GW.buttons.buttons[n].left.default.id = held
						for slot=1,22 do
							local unit = GW.spec[slot]
							GW.UI.group[slot].button[n]:EventMacroSet(Event.UI.Input.Mouse.Left.Down, GW.fixMacro(GW.makeMacro(GW.buttons.buttons[n].left),unit))
							GW.UI.group[slot].buttonBG[n]:SetTexture("Rift", item.icon)
						end
						self:SetTexture("Rift", item.icon)
						Command.Cursor(nil)
						GW.updateSetTable()
					end
				end
			end
		end, "Event.UI.Input.Mouse.Left.Up")
		
		--------------------------------------------------------------------------------------------
		-- buttonborder frame. Texture frame with nice button border and no other purpose. ---------
		--------------------------------------------------------------------------------------------
		newFrame.buttonBorder[n] = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.buttonBorder"..n, newFrame.frame)
		newFrame.buttonBorder[n]:SetWidth(GW.settings.button)
		newFrame.buttonBorder[n]:SetHeight(GW.settings.button)
		newFrame.buttonBorder[n]:SetLayer(20)
		newFrame.buttonBorder[n]:SetTexture(addon.identifier, "img/button.png")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonBorder[n]:SetPoint("TOPLEFT", newFrame.timerbar[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonBorder[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonBorder[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
		
		--------------------------------------------------------------------------------------------
		-- buttonHover frame. Texture frame for hovering highlight and no other purpose. -----------
		--------------------------------------------------------------------------------------------
		newFrame.buttonHover[n] = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.buttonHover"..n, newFrame.frame)
		newFrame.buttonHover[n]:SetWidth(GW.settings.button)
		newFrame.buttonHover[n]:SetHeight(GW.settings.button)
		newFrame.buttonHover[n]:SetLayer(25)
		newFrame.buttonHover[n]:SetTexture(addon.identifier, "img/nothing.png")
		-- attach it to the last timer (first button) or the previous button
		if n == 1 then
			newFrame.buttonHover[n]:SetPoint("TOPLEFT", newFrame.timerbar[GW.maxTimers], "BOTTOMLEFT", 0, 0)
		elseif n == (math.floor(GW.maxButtons/2)+1) then
			newFrame.buttonHover[n]:SetPoint("TOPLEFT", newFrame.buttonBG[1], "BOTTOMLEFT", 0, 0)
		else
			newFrame.buttonHover[n]:SetPoint("TOPLEFT", newFrame.buttonBG[n-1], "TOPRIGHT", 0, 0)
		end
	end
	
	local n = (math.floor(GW.maxButtons/2)+1)
	
	--target bar for display only. always full.
	newFrame.targetbar = UI.CreateFrame("Texture", "GW.UI.OPT.testFrame.targetbar", newFrame.frame)
	newFrame.targetbar:SetWidth(GW.settings.width-(2*GW.settings.padding))
	newFrame.targetbar:SetHeight(GW.settings.target)
	newFrame.targetbar:SetLayer(20)
	newFrame.targetbar:SetTexture(addon.identifier, "img/bar.png")
	newFrame.targetbar:SetBackgroundColor(1, 0, 0)
	newFrame.targetbar:SetPoint("TOPLEFT", newFrame.buttonBG[n], "BOTTOMLEFT", 0, 0)
	
	--return the new frame.
	return newFrame
end

--refreshes the currintly displayed information to reflect the current bound action to modify
function GW.updateSetTable()
	if GW.UI.SET.frame:GetVisible() == true then
		local bound = nil
		if modTable.id ~= nil then
			if modTable.action == "cast" then
				--ability is bound
				bound = GW.abilityList[modTable.id]
				if bound == nil then
					bound = Inspect.Ability.New.Detail(modTable.id)
				end
			elseif modTable.action == "use" then
				--item is bound
				bound = Inspect.Item.Detail(modTable.id)
			end
		end
		--update button texture/tooltip
		if bound ~= nil then
			GW.UI.SET.buttonBG:SetTexture("Rift", bound.icon)
		else
			GW.UI.SET.buttonBG:SetTexture(addon.identifier, "img/buttonbg.png")
		end
		
		--update target value
		if modTable.target == "@unit.pet" then
			GW.UI.SET.target:SetPosition(3)
			GW.UI.SET.targetText:SetText("Target: unit-pet")
		elseif modTable.target == "@unittt" then
			GW.UI.SET.target:SetPosition(2)
			GW.UI.SET.targetText:SetText("Target: unit-target-target")
		elseif modTable.target == "@unitt" then
			GW.UI.SET.target:SetPosition(1)
			GW.UI.SET.targetText:SetText("Target: unit-target")
		else
			GW.UI.SET.target:SetPosition(0)
			GW.UI.SET.targetText:SetText("Target: unit")
		end
	end
end